/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { SLine, SPoint, SRect } from "@saga-web/draw/lib";
import { MinDis } from "../types/MinDis";
import { Point } from "../types/Point";
import { PointToLine } from "../types/PointToLine";

export class SMathUtil {
    /**
     * 计算点到点距离
     *
     * @param   p1  第一个点
     * @param   p2  第二个点
     * @return  距离
     */
    static pointDistance(
        x1: number,
        y1: number,
        x2: number,
        y2: number
    ): number {
        return Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
    } // Function pointDistance()

    /**
     * 计算点到点集中点最小距离,返回该点和该距离
     *
     * @param   p       第一个点
     * @param   arr     点集
     * @return  距离,点坐标
     */
    static getMinDisPoint(p: SPoint, arr: Point[]): MinDis | null {
        if (!arr.length) {
            return null;
        }
        let minDis = {
            MinDis: SMathUtil.pointDistance(p.x, p.y, arr[0].X, -arr[0].Y),
            Point: arr[0]
        };
        for (let i = 1; i < arr.length; i++) {
            let ds = SMathUtil.pointDistance(p.x, p.y, arr[i].X, -arr[i].Y);
            if (ds < minDis.MinDis) {
                minDis.MinDis = ds;
                minDis.Point = arr[i];
            }
        }
        return minDis;
    } // Function getMinDisPoint()

    /**
     * 计算点到线段垂线与线段的交点
     *
     * @param   p   点
     * @param   l   线段
     * @return  距离
     */
    static pointToLine(p: SPoint, l: SLine): PointToLine {
        let d = {
            MinDis: Number.MAX_SAFE_INTEGER,
            Line: new SLine(),
            Point: new SPoint()
        };
        let bgX = Math.max(l.x1, l.x2);
        let smX = Math.min(l.x1, l.x2);
        if (l.dx == 0) {
            // l.dx为0 说明线段是垂直于x轴的
            let bgY = Math.max(l.y1, l.y2);
            let smY = Math.min(l.y1, l.y2);
            if (p.y > smY && p.y < bgY) {
                d.MinDis = Math.abs(p.x - l.x1);
                d.Line = l;
                d.Point = new SPoint(l.x1, p.y);
            }
        } else if (l.dy == 0) {
            // l.dy为0 说明线段是平行于x轴的
            if (p.x > smX && p.x < bgX) {
                d.MinDis = Math.abs(p.y - l.y1);
                d.Line = l;
                d.Point = new SPoint(p.x, l.y1);
            }
        } else {
            // 直线1
            let k1 = (l.y1 - l.y2) / (l.x1 - l.x2);
            let b1 = l.y1 - k1 * l.x1;
            // 直线2
            let k2 = -1 / k1;
            let b2 = p.y - k2 * p.x;
            // 交点
            let x = (b1 - b2) / (k2 - k1);
            let y = k1 * x + b1;
            if (x > smX && x < bgX) {
                d.MinDis = SMathUtil.pointDistance(p.x, p.y, x, y);
                d.Line = l;
                d.Point = new SPoint(x, y);
            }
        }
        return d;
    } // Function pointDistance()

    /**
     * 计算点到点集中线段最小距离,返回该点和该距离
     *
     * @param   p       第一个点
     * @param   arr     点集
     * @return  距离,点坐标
     */
    static getMinDisLine(p: SPoint, arr: Point[]): PointToLine | null {
        if (arr.length < 2) {
            return null;
        }
        let PTL = SMathUtil.pointToLine(
            p,
            new SLine(
                arr[arr.length - 1].X,
                -arr[arr.length - 1].Y,
                arr[0].X,
                -arr[0].Y
            )
        );
        for (let i = 0; i < arr.length - 1; i++) {
            let temp = SMathUtil.pointToLine(
                p,
                new SLine(arr[i].X, -arr[i].Y, arr[i + 1].X, -arr[i + 1].Y)
            );
            if (temp.MinDis < PTL.MinDis) {
                PTL = temp;
            }
        }
        return PTL;
    } // Function getMinDisPoint()

    /**
     * 计算矩形是否有交集(外包矩形算法)
     *
     * @param   rect1   矩形1
     * @param   rect2   矩形2
     * @return  boolean
     * */
    static rectIntersection(rect1: SRect, rect2: SRect): boolean {
        let minX = rect1.x < rect2.x ? rect1.x : rect2.x;
        let minY = rect1.y < rect2.y ? rect1.y : rect2.y;
        let maxX = rect1.right > rect2.right ? rect1.right : rect2.right;
        let maxY = rect1.bottom > rect2.bottom ? rect2.bottom : rect2.bottom;
        return (
            rect1.width + rect2.width > maxX - minX &&
            rect1.height + rect2.height > maxY - minY
        );
    } // Function rectIntersection()
} // Class SMathUtil
